/********************************************************************************
 *
 *  FILE NAME   :   sample.c
 *  CREATE      :   2007.10.29
 *  MCU         :   M32C/85
 *  VERSION     :   1.00
 *  DESCRIPTION :   
 *
 *==history======================================================================

********************************************************************************/

/********************************************************************************
 *      Include Section                                                         *
 ********************************************************************************/
/* M32C/85 SFR */
#include "sfr32c85.h"
/* LIN Sleve */
#include "slin.h"
#include "slin_def.h"
#include "slin_frm.h"
/* For Sample */
#include "sample.h"

/*** FUNCTION prototype ***/
void init_sfr(void);
char get_slave_id(void);
void disp_led(char m_val);
void blink_led(void);
int get_sw3(void);
int get_sw4(void);
void set_sys_timer(int);
int chk_sys_timer(void);
void stop_mcu(void);

unsigned char counter=0;


/*** VARIABLE prototype ***/
l_u8 info_data_buf[8];
l_u8 led_data_buf[8];

/*>>*+*********************************************************************/
void main(void);
void main()
/*------------------------------------------------------------------------
 Description:
   main routine of this program.
--------------------------------------------------------------------------
 Input:     no
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    int lp = 0;

    /*** Initialization SFR ***/
    init_sfr();

    /*** Initialization main timer ***/
    set_sys_timer(SYSTEM_TIME);

    /*** Initialization LIN S/W ***/
    if(l_sys_init() == E_NG){
        return;
    }
    else{
        l_ifc_init();
    }
    /* set master ID */
    lb_masterid = 0;
    /* set slave ID */
    lb_slaveid = get_slave_id();
    /* initial LIN */
    do{
    }while(l_ifc_connect() == E_NG);

    /*** main roop ***/
    for(;;++lp){

        /* wait for system timer */
        while(!chk_sys_timer());
        switch(lb_state){
        /* Reset */
        case LIN_ST_RESET:
            if(get_sw3()){
                l_ifc_connect();
                l_wakeup();
            }
            break;

        /* Sleep */
        case LIN_ST_SLEEP:
            if(get_sw3()){
                l_wakeup();
            }
            if(get_sw4()){
                l_ifc_disconnect();
            }
            break;

        /* Run */
        case LIN_ST_RUN:
            l_set_rcvreq(led_frm);
            l_set_sndreq(info_frm1);
            info_data_buf[0] = lb_slaveid;
            info_data_buf[1] = counter;
            l_set_snddata(info_frm1, info_data_buf);

            if(lb_rcv_comp){
                if(l_read_rcvsts(led_frm) == LIN_STS_END){
                    if(l_get_rcvdata(led_frm, led_data_buf)==E_OK){
                        disp_led(led_data_buf[1]);
                    }
                    l_clr_rcvsts(led_frm);
                }
            }
            break;
        }

        if(lp >= (1000/SYSTEM_TIME)){
            if((lb_state==LIN_ST_RUN) && (lb_no_bus_activity==LIN_ERR))
                blink_led();
            counter++;
            lp = 0;
        }

    }
}

/*>>*+*********************************************************************/
void init_sfr(void)
/*------------------------------------------------------------------------
 Description:
   initialization SFR.
--------------------------------------------------------------------------
 Input:     no
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    /*** set processor mode ***/
    /*** set clock  8MHz * 8 / 2  PLL32MHz   ***/
    prc0 = 1;
    cm05 = 0;
    asm("nop");
    asm("nop");
    asm("nop");
    mcd = 0x12;
    cm07 = 0;
    cm21 = 0;
    cm04 = 0;
    plc = 0x0254;
    plc = 0x02d4;
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    cm1 = 0xa0;
    prcr = 0x00;

    /*** set port directin ***/
    pd2 = 0x00;
    pd3 = 0x0f;
    pd4 = 0xff;
    pd5_6 = 1;

    /*** set port value ***/
    p3_0 = 0;
    p3_1 = 1;
    p3_2 = 1;
    p3_3 = 1;
    p4 = 0x00;
    p5_6 = 1;   /* NSLP */

    asm("FSET I");
}


/*>>*+*********************************************************************/
char get_slave_id(void)
/*------------------------------------------------------------------------
 Description:
   read slave id value by using SW2 on this board
--------------------------------------------------------------------------
 Input:     no
 Output:    (char) slave id
 After:     no
 Note:      no
***********************************************************************<<*/
{
    char i;
    char m_id = 0;

    m_id = ((~PORT_SLAVE_ID) & 3) + 1;

    return m_id;

}


/*>>*+*********************************************************************/
void disp_led(char m_val)
/*------------------------------------------------------------------------
 Description:
   display LED
--------------------------------------------------------------------------
 Input:     (char)LED value
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    char val;

    val = PORT_LED_VALUE;
    val &= ~PORT_LED_MASK;
    val |= (m_val&PORT_LED_MASK);

    PORT_LED_VALUE = val;
}


/*>>*+*********************************************************************/
void blink_led(void)
/*------------------------------------------------------------------------
 Description:
   blink LED which means main program is running.
--------------------------------------------------------------------------
 Input:     no
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    PORT_LED_RUN = ~PORT_LED_RUN;
}


/*>>*+*********************************************************************/
int get_sw3(void)
/*------------------------------------------------------------------------
 Description:
   get SW3 information
--------------------------------------------------------------------------
 Input:     no
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    int result;

    result = PORT_SW3 ? 0:1;
    return result;
}


/*>>*+*********************************************************************/
int get_sw4(void)
/*------------------------------------------------------------------------
 Description:
   get SW4 information
--------------------------------------------------------------------------
 Input:     no
 Output:    no
 After:     no
 Note:      no
***********************************************************************<<*/
{
    int result;

    result = PORT_SW4 ? 0:1;
    return result;
}


/**************************************************************************/
void set_sys_timer(int in_time)
/*------------------------------------------------------------------------*/
/* Description:                                                           */
/*     circular time for main routine                                     */
/*------------------------------------------------------------------------*/
/* Input:  (int)in_time[ms]                                               */
/* Output:                                                                */
/**************************************************************************/
{
    /*** Disable interrupt ***/
    asm("pushc flg");
    asm("fclr i");

    /*** TimerB2 setting = 1[ms] ***/
    tb2s = 0;
    tb2ic = 0;
    tb2mr = 0x00;
    tb2 = 32000;

    /*** TimerA4 setting = in_time[ms] */
    ta4s = 0;
    ta4ic = 0;
    ta4mr = 0x01;
    ta4tgl = 1;
    ta4tgh = 0;
    ta4 = in_time -1;

    /*** Enable interrupt ***/
    asm("popc flg");

    /*** TimerA4/B2 start counting */
    ta4s = 1;
    tb2s = 1;
}

/**************************************************************************/
int chk_sys_timer()
/*------------------------------------------------------------------------*/
/* Description:                                                           */
/*     Check system Timer                                                 */
/*------------------------------------------------------------------------*/
/* Input:                                                                 */
/* Output: [0] not yet                                                    */
/*         [1] OK                                                         */
/**************************************************************************/
{
    if(ir_ta4ic == 1){
        ir_ta4ic = 0;
        return 1;
    }
    else{
        return 0;
    }
}



/***** End of File *****/